# PAX - the Pythonic API for XML
# developed by Lalo Martins <lalo@laranja.org>
# this file is placed on public domain

import core
from backwards_compatibility import *

# get a namespace, if provided
def split_ns(spec, default=None):
    if ' ' in spec:
        return spec.split(' ', 1)
    else:
        return default, spec

class _parse_in_progress(object):
    def __init__(self, parser):
        self.stack = []
        self.namespaces = []
        self.ns_pushed = False
        self.ns_popped = False
        self.parser = parser
        parser.StartElementHandler = self.start_element
        parser.EndElementHandler = self.end_element
        parser.ProcessingInstructionHandler = self.processing_instruction
        parser.CharacterDataHandler = self.text
        parser.StartNamespaceDeclHandler = self.add_namespace
        parser.EndNamespaceDeclHandler = self.pop_namespace
        parser.CommentHandler = self.comment
        parser.returns_unicode = True
        parser.buffer_text = True

    def __raw(self):
        return self.parser.GetInputContext().split('>')[0] + u'>'

    def start_element(self, name, raw_attributes):
        ns, name = split_ns(name)
        attributes = {}
        for aspec, avalue in raw_attributes.items():
            ans, aname = split_ns(aspec, ns)
            attributes.setdefault(ans, {})[aname] = avalue
        if self.ns_pushed:
            nsdecls = self.namespaces[0]
        else:
            nsdecls = {}
        self.stack.append(core.Element(ns, name, attributes, nsdecls, self.__raw()))
        self.ns_pushed = False

    def end_element(self, name):
        if len(self.stack) > 1:
            self.stack[-2].append(self.stack[-1])
        else:
            self.result = self.stack[-1]
        self.stack.pop()
        self.ns_popped = False

    def processing_instruction(self, *a):
        self.stack[-1].append(core.Declaration(self.__raw()[2:-1]))

    def text(self, data):
        self.stack[-1].append(data)

    def add_namespace(self, prefix, url):
        if self.ns_pushed:
            d = self.namespaces[0]
        else:
            self.ns_pushed = True
            d = {}
            self.namespaces.insert(0, d)
        d[prefix] = url

    def pop_namespace(self, prefix):
        if not self.ns_popped:
            del self.namespaces[0]
        self.ns_popped = True
    
    def comment(self, data):
        self.stack[-1].append(core.Comment(data))


import xml.parsers.expat

def parse(data, builtin_namespaces=None, encoding=None):
    parser = xml.parsers.expat.ParserCreate(namespace_separator=' ')
    handler = _parse_in_progress(parser)
    if builtin_namespaces is not None:
        handler.namespaces.append(builtin_namespaces)
    parser.Parse(data, 1)
    return handler.result
